<?php
/**
* This file is part of the Agora-Project Software package.
*
* @copyright (c) Agora-Project Limited <https://www.agora-project.net>
* @license GNU General Public License, version 2 (GPL-2.0)
*/


/*
 * Modele des agendas
 */
class MdlCalendar extends MdlObject
{
	//Propriétés principales
	const moduleName="calendar";
	const objectType="calendar";
	const dbTable="ap_calendar";
	const MdlObjectContent="MdlCalendarEvent";
	//Propriétés d'IHM
	const hasShortcut=false;
	const hasNotifMail=false;
	//Droit de supprimer l'agenda perso. True si suppression de l'user correspondant
	public static $persoCalendarDeleteRight=false;
	//Champs obligatoires, Champs de recherche et Champs de tri d'affichage
	public static $requiredFields=array("title");
	public static $searchFields=array("title","description");
	//Valeurs en cache (calculées qu'1 fois)
	private static $_visibleCalendars=null;
	private static $_visiblePersoCalendars=null;
	private static $_affectationCalendars=null;


	/*
	 * SURCHARGE : Constructeur
	 */
	function __construct($objIdOrValues=null)
	{
		parent::__construct($objIdOrValues);
		//Libellé de l'agenda perso = nom de l'utilisateur correspondant
		if($this->type=="user")	{$this->title=$this->displayAutor();}
		//Plage horaire de l'agenda
		if(empty($this->timeSlot)){
			$this->timeSlotBegin=8;
			$this->timeSlotEnd=20;
		}else{
			$tmpTimeSlot=explode("-",$this->timeSlot);
			$this->timeSlotBegin=$tmpTimeSlot[0];
			$this->timeSlotEnd=$tmpTimeSlot[1];
		}
	}

	/*
	 * Verifie si c'est l'agenda perso de l'user courant
	 */
	function isMyPerso()
	{
		return ($this->type=="user" && $this->isAutor());
	}

	/*
	 * SURCHARGE : Privilège de droits d'accès pour l'admin d'espace : pas sur les agendas persos
	 */
	public function accessRightAdminSpacePrivilege()
	{
		return ($this->type!="user");
	}

	/*
	 * SURCHARGE : Droit d'ajouter du contenu dans l'objet conteneur. Pour les "guests" : uniquement des propositions d'evt
	 */
	public function editContentRight()
	{
		return (Ctrl::$curUser->isUser())  ?  parent::editContentRight()  :  false;
	}

	/*
	 * SURCHARGE : Droit d'ajouter un agenda (partagé)
	 */
	public static function addRight()
	{
		return (Ctrl::$curUser->isAdminCurSpace() || (Ctrl::$curUser->isUser() && Ctrl::$curSpace->moduleOptionEnabled("dashboard","ajout_agenda_ressource_admin")==false));
	}
	
	/*
	 * SURCHARGE : droit de suppression d'un agenda : pas pour les agendas d'users
	 */
	public function deleteRight(){
		return ($this->type=="user" && $this::$persoCalendarDeleteRight==false)  ?  false  :  parent::deleteRight();
	}

	/*
	 * SURCHARGE : suppression d'agenda
	 */
	public function delete()
	{
		//Controle le droit d'accès
		if($this->deleteRight())
		{
			//Supprime les evenements affectés uniquement à l'agenda en question
			$eventList=Db::getCol("SELECT DISTINCT _idEvt FROM ap_calendarEventAffectation WHERE _idCal=".$this->_id." AND _idEvt NOT IN (select _idEvt from ap_calendarEventAffectation where _idCal!=".$this->_id.")");
			foreach($eventList as $_idEvt){
				$tmpEvt=Ctrl::getObj("calendarEvent",$_idEvt);
				$tmpEvt->delete();
			}
			//Supprime les jointures de l'agenda avec les evenements
			Db::query("DELETE FROM ap_calendarEventAffectation WHERE _idCal=".$this->_id);
			//Supprime enfin l'agenda
			parent::delete();
		}
	}

	/*
	 * Evenements de l'agenda (confirmés)
	 */
	public function evtList($curBegin=null, $curEnd=null, $accessRightMini=0.5, $orderByHourMinute=true, $pluginParams=null)
	{
		////	Evt sur une période/créneau donnée? Attention car tous les evt périodiques sont récupérés : pour l'affichage, il faudra les filtrer avec "evtFilterDays()", qui fait appel à "evtFilterDay()"
		$sqlTimeSlot=null;
		if(!empty($curBegin) && !empty($curEnd)){
			$tmpBegin=Db::format(date("Y-m-d 00:00",$curBegin));
			$tmpEnd=Db::format(date("Y-m-d 23:59",$curEnd));
			$sqlTimeSlot="AND (  (dateBegin between ".$tmpBegin." and ".$tmpEnd.")  OR  (dateEnd between ".$tmpBegin." and ".$tmpEnd.")  OR  (DateBegin <= ".$tmpBegin." and DateEnd >= ".$tmpEnd.")  OR  periodType is not null)";
		}
		////	 Liste des evenements, en fonction des droits d'accès. Tri par "Heure:Minute" si affiché sur un jour (cf. evt périodiques) OU Tri par "dateBegin" si affiché une liste complete (cf. plugins)
		$sqlPlugins=(!empty($pluginParams))  ?  "AND ".MdlCalendarEvent::sqlPluginObjects($pluginParams)  :  null;//Sélection d'un plugin?
		$sqlOrderBy=($orderByHourMinute==true)  ?  "DATE_FORMAT(dateBegin,'%H:%i') ASC"  :  "dateBegin ASC";//Filtre par "H:m" ou par "dateBegin"
		$eventsObjList=Db::getObjTab("calendarEvent","SELECT * FROM ap_calendarEvent WHERE _id IN (select _idEvt from ap_calendarEventAffectation where _idCal=".$this->_id." and confirmed=1) ".$sqlTimeSlot." ".$sqlPlugins." ORDER BY ".$sqlOrderBy);
		////	renvoie les evts en fonction du droit d'accès minimum 
		$evtListReturned=[];
		foreach($eventsObjList as $tmpObj){
			if($tmpObj->accessRight()>=$accessRightMini)  {$evtListReturned[]=$tmpObj;}
		}
		//renvoie les evenements
		return $evtListReturned;
	}

	/*
	 * Controle si un evt est sur une période/créneau donné
	 */
	public static function evtInTimeSlot($evtBegin, $evtEnd, $curBegin, $curEnd)
	{
		//Retourne true :  Début de l'evt dans le créneau  ||  Fin de l'evt dans le créneau  ||  evt avant et après le créneau
		return ($evtBegin>=$curBegin && $evtBegin<=$curEnd) || ($evtEnd>=$curBegin && $evtEnd<=$curEnd) || ($evtBegin<=$curBegin && $evtEnd>=$curEnd);
	}

	/*
	 * Filtre des Evts pour une journée ou un créneau horaire dans une journée
	 */
	public static function evtFilterDay($evtList, $curBegin, $curEnd)
	{
		$evtListReturned=[];
		foreach($evtList as $tmpEvt)
		{
			//Clone l'evt pour chaque jour (evt sur plusieurs jours ou périodique : une instance d'evt par jour)  &&  Debut/fin de l'evt au format timestamp 
			$tmpEvt=clone $tmpEvt;
			$evtBegin=strtotime($tmpEvt->dateBegin);
			$evtEnd=strtotime($tmpEvt->dateEnd);
			//EVT SUR LA JOURNEE/CRENEAU HORAIRE : Début de l'evt dans le créneau || Fin de l'evt dans le créneau || evt avant et après le créneau
			if(static::evtInTimeSlot($evtBegin,$evtEnd,$curBegin,$curEnd))	{$evtListReturned[]=$tmpEvt;}
			//EVT PERIODIQUE
			elseif(!empty($tmpEvt->periodType))
			{
				//Evenement sur le jour =>  déjà commencé  &&  (pas de fin de périodicité || fin de périodicité pas encore arrivé)  &&  (pas de date d'exception || "dateBegin" absent des dates d'exception)
				if($evtBegin<$curBegin  &&  (empty($tmpEvt->periodDateEnd) || $curEnd<=strtotime($tmpEvt->periodDateEnd." 23:59"))  &&  (empty($tmpEvt->periodDateExceptions) || in_array(date("Y-m-d",$curBegin),Txt::txt2tab($tmpEvt->periodDateExceptions))==false))
				{
					//L'evt périodique est présent sur le jour courant : Reformate le début/fin de l'evt pour qu'il corresponde à la date courante
					$periodValues=Txt::txt2tab($tmpEvt->periodValues);
					$dateFormatModif=$dateFormatConserved=null;
					if($tmpEvt->periodType=="weekDay" && in_array(date("N",$curBegin),$periodValues))												{$dateFormatModif="Y-m-d";	$dateFormatConserved=" H:i";}//jour de semaine
					elseif($tmpEvt->periodType=="monthDay" && in_array(date("d",$curBegin),$periodValues))											{$dateFormatModif="Y-m-d";	$dateFormatConserved=" H:i";}//jour du mois
					elseif($tmpEvt->periodType=="month" && in_array(date("m",$curBegin),$periodValues) && date("d",$evtBegin)==date("d",$curBegin))	{$dateFormatModif="Y-m";	$dateFormatConserved="-d H:i";}//mois
					elseif($tmpEvt->periodType=="year" && date("m-d",$evtBegin)==date("m-d",$curBegin))												{$dateFormatModif="Y";		$dateFormatConserved="-m-d H:i";}//année
					//Reformate pour que le début/fin de l'evt corresponde au jour
					if(!empty($dateFormatModif) && !empty($dateFormatConserved))
					{
						$tmpEvt->dateBegin=date($dateFormatModif,$curBegin).date($dateFormatConserved,$evtBegin);
						$tmpEvt->dateEnd  =date($dateFormatModif,$curEnd).date($dateFormatConserved,$evtEnd);
						$evtBegin=strtotime($tmpEvt->dateBegin);
						$evtEnd=strtotime($tmpEvt->dateEnd);
						//Ajoute l'evt s'il est bien sur le créneau courant (cf. controles de créneaux horaires occupés..)
						if(static::evtInTimeSlot($evtBegin,$evtEnd,$curBegin,$curEnd))	{$evtListReturned[]=$tmpEvt;}
					}
				}
			}
		}
		return $evtListReturned;
	}

	/*
	 * Filtre des Evts pour une période de plusieurs jours
	 */
	public static function evtFilterDays($evtList, $curBegin, $curEnd)
	{
		$evtListReturned=[];
		for($timeDay=$curBegin+43200; $timeDay<=$curEnd; $timeDay+=86400){//12h de décalage (pour les heures d'été/hivers)
			$subPeriodBegin=strtotime(date("Y-m-d",$timeDay)." 00:00");
			$subPeriodEnd=strtotime(date("Y-m-d",$timeDay)." 23:59");
			$evtListReturned=array_merge($evtListReturned, static::evtFilterDay($evtList,$subPeriodBegin,$subPeriodEnd));
		}
		return array_unique($evtListReturned,SORT_REGULAR);//"SORT_REGULAR" pour les objets
	}

	/*
	 * Droit de confirmer une proposition d'événement?
	 */
	public function confirmEventPropositionRight()
	{
		foreach(self::myCalendars() as $tmpCalendar){
			if($tmpCalendar->_id==$this->_id)	{return true;}
		}
	}

	/*
	 * Agendas visibles pour l'user courant
	 * => agendas de ressource ET agenda persos activés qui nous sont affectés (& celui de l'user courant)
	 */
	public static function visibleCalendars()
	{
		if(self::$_visibleCalendars===null){
			$sqlDisplayedObjects=self::sqlDisplayedObjects();
			self::$_visibleCalendars=Db::getObjTab("calendar","SELECT DISTINCT * FROM ap_calendar WHERE  (type='ressource' AND ".$sqlDisplayedObjects.")  OR  (type='user' AND _idUser not in (select _id from ap_user where calendarDisabled=1) AND (_idUser=".Ctrl::$curUser->_id." or ".$sqlDisplayedObjects."))");
			self::$_visibleCalendars=self::sortCalendars(self::$_visibleCalendars);//Tri les agendas
		}
		return self::$_visibleCalendars;
	}

	/*
	 * Agendas de l'user courant : agenda perso & agendas de ressource accessible en écriture (aussi en lecture?)
	 * exple: pour le dashboard et les propositions d'evt
	 */
	public static function myCalendars()
	{
		if(self::$_visiblePersoCalendars===null)
		{
			self::$_visiblePersoCalendars=[];
			foreach(self::visibleCalendars() as $tmpCalendar){
				if($tmpCalendar->isMyPerso() || ($tmpCalendar->type=="ressource" && $tmpCalendar->fullRight()))
					{self::$_visiblePersoCalendars[]=$tmpCalendar;}
			}
		}
		return self::$_visiblePersoCalendars;
	}

	/*
	 * Agendas sur lesquels l'users courant peut affecter/proposer des événements
	 * => "visibleCalendars()" + autres agendas de l'espace courant, pour les propositions d'evenement
	 */
	public static function affectationCalendars()
	{
		if(self::$_affectationCalendars===null)
		{
			self::$_affectationCalendars=self::visibleCalendars();
			//Uniquement pour les users (pas guests) : ajoute les agendas des users de l'espace courant (propositions d'evt)
			if(Ctrl::$curUser->isUser()){
				$userIds="0,".implode(",",Ctrl::$curSpace->getUsers("ids"));
				$usersCalendars=Db::getObjTab("calendar","SELECT DISTINCT * FROM ap_calendar WHERE _idUser IN (".trim($userIds,",").") AND type='user' AND _idUser NOT IN (select _id from ap_user where calendarDisabled=1)");
				foreach($usersCalendars as $tmpCalendar){
					if(!in_array($tmpCalendar,self::$_affectationCalendars))	{self::$_affectationCalendars[]=$tmpCalendar;}
				}
				self::$_affectationCalendars=self::sortCalendars(self::$_affectationCalendars);//Tri les agendas
			}
		}
		return self::$_affectationCalendars;
	}

	/*
	 * Agendas Affichés actuellement
	 */
	public static function displayedCalendars($visibleCalendars)
	{
		//init
		$displayedCalendars=[];
		$prefDisplayedCalendars=Txt::txt2tab(Ctrl::prefUser("displayedCalendars"));//Agendas affichés, enregistrés en préférence?
		//Ajoute l'agenda si :  l'agenda est dans les préférences d'affichage ou c'est mon agenda perso (agenda par défaut)
		foreach($visibleCalendars as $tmpCalendar){
			if(in_array($tmpCalendar->_id,$prefDisplayedCalendars) || empty($prefDisplayedCalendars) && $tmpCalendar->isMyPerso())  {$displayedCalendars[]=$tmpCalendar;}
		}
		//Pas d'agendas affiché : prend le premier de la liste
		if(empty($displayedCalendars) && !empty($visibleCalendars))  {$displayedCalendars[]=$visibleCalendars[0];}
		//retour le résultat
		return $displayedCalendars;
	}

	/*
	 * Tri d'une liste d'objets calendriers : en fonction du type, puis du titre ET met l'agenda de l'user courant en 1er
	 */
	public static function sortCalendars($calendarsTab)
	{
		//Créé une liste d'agendas partagés et persos
		$myCal=$usersCals=$ressourceCals=[];
		foreach($calendarsTab as $tmpCalendar){
			if($tmpCalendar->isMyPerso())		{$myCal[]=$tmpCalendar;}
			elseif($tmpCalendar->type=="user")	{$usersCals[]=$tmpCalendar;}
			else								{$ressourceCals[]=$tmpCalendar;}
		}
		//Tri des agendas : puis les agendas partagés -> mon agenda -> puis les autres agendas perso
		usort($usersCals,["self","sortCompareCalendars"]);
		usort($ressourceCals,["self","sortCompareCalendars"]);
		return array_merge($ressourceCals, $myCal, $usersCals);
	}
	public static function sortCompareCalendars($obj1, $obj2)
	{
		return strcmp($obj1->title, $obj2->title);
	}

	/*
	 * SURCHARGE : Menu contextuel
	 */
	public function menuContext($options=null)
	{
		//Accès en édition?
		if($this->editRight())
		{
			//Export d'agenda (téléchargement / envoi par mail)
			$urlExportIcs="?ctrl=calendar&action=exportEvents&targetObjId=".$this->_targetObjId;
			$options["specificOptions"][]=["inMainMenu"=>true,"actionJs"=>"redir('".$urlExportIcs."')","iconSrc"=>"app/img/export.png","label"=>Txt::trad("CALENDAR_exporter_ical")];
			$options["specificOptions"][]=["inMainMenu"=>true,"actionJs"=>"redir('".$urlExportIcs."&sendMail=true')","iconSrc"=>"app/img/mail.png","label"=>Txt::trad("CALENDAR_exporter_ical_mail"),"tooltip"=>Txt::trad("CALENDAR_exporter_ical_mail2")."<br>".Txt::trad("envoyer_a")." ".Ctrl::$curUser->mail];
			//Import d'événements ou suppression des d'anciens événements dans l'agenda
			$options["specificOptions"][]=["inMainMenu"=>true,"actionJs"=>"lightboxOpen('?ctrl=calendar&action=importEvents&targetObjId=".$this->_targetObjId."')","iconSrc"=>"app/img/import.png","label"=>Txt::trad("CALENDAR_importer_ical")];
			$options["specificOptions"][]=["inMainMenu"=>true,"actionJs"=>"confirmRedir('".Txt::trad("CALENDAR_confirm_suppr_anciens_evt",true)."','?ctrl=calendar&action=oldEvtDelete&targetObjId=".$this->_targetObjId."')","iconSrc"=>"app/img/calendar/oldEvtDelete.png","label"=>Txt::trad("CALENDAR_suppr_anciens_evt"),"tooltip"=>Txt::trad("CALENDAR_suppr_anciens_evt_info")];
		}
		//Renvoie le menu surchargé
		return parent::menuContext($options);
	}
}